#!/usr/bin/env python
#-*- coding:utf-8 -*-
# Copyright 2012 Tero Karvinen http://TeroKarvinen.com
"""
Show window, or run program if it has no window. 

Copyright 2012 Tero Karvinen http://TeroKarvinen.com
GNU General Public License, version 2

Examples:

showrun xclock

showrun /usr/bin/xclock
"""
__version__="0.3.9"
__author__="Tero Karvinen <teropkg@gmx.com>"
__copyright__="Copyright 2012 Tero Karvinen http://TeroKarvinen.com"
__license__="GNU General Public License, version 2"

import wnck, sys, psutil, os, os.path, argparse, subprocess, time

def showRun(cmd, verbose):
	screen = wnck.screen_get_default()
	screen.force_update()

	try:
		cmdwhich=subprocess.check_output("which '%s'" % cmd, shell=True)
		cmdwhich=cmdwhich.strip()
		cmdwhich=os.path.realpath(cmdwhich)
		print "which command: '%s'" % cmdwhich
	except subprocess.CalledProcessError as e:
		cmdwhich="CMD_NOT_IN_PATH"
		print "W: Command not in path. ", e

	interpreters=["bash", "guile", "nickle", "perl", "python", "python2.7", "python3", "python3.2", "ruby", "ruby1.8", "sh", "wish"]	# TODO: other versions

	for w in screen.get_windows():
		name=w.get_name(), 
		app=w.get_application()
		pid=w.get_pid()
		if pid<1:
			print "W: pid<1, still trying... "
			pid=app.get_pid()
		p = psutil.Process(pid)
		if verbose:
			print '%s(%i) %s "%s"' % (p.exe, pid, p.name, name)
		if cmdwhich == p.exe or cmd == p.name:
			print 'Found window. Showing "%s"' % name
			w.activate(int(time.time()))
			return True
		s=os.path.basename(p.cmdline[0])
		if s in interpreters:
			for s in p.cmdline[1:]:
				if s.startswith("-"): # eg. 'python -tt ...'
					continue
				if cmdwhich == s or cmd == s:
					print 'Found script window. Showing "%s"' % name
					w.activate(int(time.time()))
					return True

	realcmd="%s &" % cmd
	print 'No window found. Starting "%s"' % realcmd
	os.system(realcmd)

def main():
	parser = argparse.ArgumentParser(
		description=__doc__, 
		formatter_class=argparse.RawTextHelpFormatter # for newlines in description
	)
	parser.add_argument("command")
	parser.add_argument("--verbose", "-v",	 action="store_true")
	parser.add_argument('--version', action='version', version=__version__)
	args = parser.parse_args()
	showRun(args.command, args.verbose)

if __name__ == '__main__':
	sys.exit(main())


